/*
    avicore
    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef __BITMAP_H__
#define __BITMAP_H__


#include "common.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define BI_COMP_RGB       0
#define BI_COMP_RLE8      1
#define BI_COMP_RLE4      2
#define BI_COMP_BITFIELDS 3


#define BMFH_SIZE 14
#define BMIH_SIZE 40
#define RGBQUAD_SIZE 4


#ifdef USE_GCC


typedef struct _BitmapFileHeader
{
  guint16 type __attribute__((packed));
  guint32 size __attribute__((packed));
  guint16 reserved1 __attribute__((packed));
  guint16 reserved2 __attribute__((packed));
  guint32 off_bits __attribute__((packed));
} BitmapFileHeader;


typedef struct _BitmapInfoHeader
{
  guint32 size __attribute__((packed));
  gint32 width __attribute__((packed));
  gint32 height __attribute__((packed));
  guint16 planes __attribute__((packed));
  guint16 bit_count __attribute__((packed));
  guint32 compression __attribute__((packed));
  guint32 size_image __attribute__((packed));
  gint32 x_pixels_per_meter __attribute__((packed));
  gint32 y_pixels_per_meter __attribute__((packed));
  guint32 color_used __attribute__((packed));
  guint32 color_important __attribute__((packed));
} BitmapInfoHeader;


typedef struct _RgbQuad
{
  guint8 blue __attribute__((packed));
  guint8 green __attribute__((packed));
  guint8 red __attribute__((packed));
  guint8 reserved __attribute__((packed));
} RgbQuad;


#else /* not USE_GCC */


typedef struct _BitmapFileHeader { guint8 dummy[BMFH_SIZE]; } BitmapFileHeader;
typedef struct _BitmapInfoHeader { guint8 dummy[BMIH_SIZE]; } BitmapInfoHeader;
typedef struct _RgbQuad { guint8 dummy[RGBQUAD_SIZE]; } RgbQuad;


#endif /* not USE_GCC */


/******************************************************************************
*                                                                             *
* ja:ビットマップ構造体マクロ                                                 *
*                                                                             *
******************************************************************************/
#define bmfh_get_type(bmfh) GUINT16_FROM_LE(*(guint16 *)(bmfh))
#define bmfh_get_size(bmfh) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(bmfh)+2))
#define bmfh_get_off_bits(bmfh)                                             \
                            GUINT32_FROM_LE(*(guint32 *)((guint8 *)(bmfh)+10))
#define bmfh_set_type(bmfh,type) (*(guint16 *)(bmfh)=GUINT16_TO_LE(type))
#define bmfh_set_size(bmfh,size)                                            \
                        (*(guint32 *)((guint8 *)(bmfh)+2)=GUINT32_TO_LE(size))
#define bmfh_set_off_bits(bmfh,off_bits)                                    \
                    (*(guint32 *)((guint8 *)(bmfh)+10)=GUINT32_TO_LE(off_bits))


#define bmih_get_size(bmih) GUINT32_FROM_LE(*(guint32 *)(bmih))
#define bmih_get_width(bmih) GINT32_FROM_LE(*(gint32 *)((guint8 *)(bmih)+4))
#define bmih_get_height(bmih) GINT32_FROM_LE(*(gint32 *)((guint8 *)(bmih)+8))
#define bmih_get_planes(bmih)                                               \
                            GUINT16_FROM_LE(*(guint16 *)((guint8 *)(bmih)+12))
#define bmih_get_bit_count(bmih)                                            \
                            GUINT16_FROM_LE(*(guint16 *)((guint8 *)(bmih)+14))
#define bmih_get_compression(bmih)                                          \
                            GUINT32_FROM_LE(*(guint32 *)((guint8 *)(bmih)+16))
#define bmih_get_size_image(bmih)                                           \
                            GUINT32_FROM_LE(*(guint32 *)((guint8 *)(bmih)+20))
#define bmih_get_x_pixels_per_meter(bmih)                                   \
                            GINT32_FROM_LE(*(gint32 *)((guint8 *)(bmih)+24))
#define bmih_get_y_pixels_per_meter(bmih)                                   \
                            GINT32_FROM_LE(*(gint32 *)((guint8 *)(bmih)+28))
#define bmih_get_color_used(bmih)                                           \
                            GUINT32_FROM_LE(*(guint32 *)((guint8 *)(bmih)+32))
#define bmih_get_color_important(bmih)                                      \
                            GUINT32_FROM_LE(*(guint32 *)((guint8 *)(bmih)+36))
#define bmih_set_size(bmih,size) (*(guint32 *)(bmih)=GUINT32_TO_LE(size))
#define bmih_set_width(bmih,width)                                          \
                        (*(gint32 *)((guint8 *)(bmih)+4)=GINT32_TO_LE(width))
#define bmih_set_height(bmih,height)                                        \
                        (*(gint32 *)((guint8 *)(bmih)+8)=GINT32_TO_LE(height))
#define bmih_set_planes(bmih,planes)                                        \
                    (*(guint16 *)((guint8 *)(bmih)+12)=GUINT16_TO_LE(planes))
#define bmih_set_bit_count(bmih,bit_count)                                  \
                (*(guint16 *)((guint8 *)(bmih)+14)=GUINT16_TO_LE(bit_count))
#define bmih_set_compression(bmih,compression)                              \
                (*(guint32 *)((guint8 *)(bmih)+16)=GUINT32_TO_LE(compression))
#define bmih_set_size_image(bmih,size_image)                                \
                (*(guint32 *)((guint8 *)(bmih)+20)=GUINT32_TO_LE(size_image))
#define bmih_set_x_pixels_per_meter(bmih,x_pixels_per_meter)                \
            (*(gint32 *)((guint8 *)(bmih)+24)=GINT32_TO_LE(x_pixels_per_meter))
#define bmih_set_y_pixels_per_meter(bmih,y_pixels_per_meter)                \
            (*(gint32 *)((guint8 *)(bmih)+28)=GINT32_TO_LE(y_pixels_per_meter))
#define bmih_set_color_used(bmih,color_used)                                \
                (*(guint32 *)((guint8 *)(bmih)+32)=GUINT32_TO_LE(color_used))
#define bmih_set_color_important(bmih,color_important)                      \
            (*(guint32 *)((guint8 *)(bmih)+36)=GUINT32_TO_LE(color_important))


#define RGB_BLUE 0
#define RGB_GREEN 1
#define RGB_RED 2
#define RGB_RESERVED 3
#define rgbquad_get(rgb,index,color) (*((guint8 *)(rgb)+(index)*4+(color)))
#define rgbquad_set(rgb,index,color,value)                                  \
                                (*((guint8 *)(rgb)+(index)*4+(color))=(value))
#define rgbquad_set_all(rgb,index,blue,green,red,reserved) (                \
                        *((guint8 *)(rgb)+(index)*4+RGB_BLUE)=(blue),       \
                        *((guint8 *)(rgb)+(index)*4+RGB_GREEN)=(green),     \
                        *((guint8 *)(rgb)+(index)*4+RGB_RED)=(red),         \
                        *((guint8 *)(rgb)+(index)*4+RGB_RESERVED)=(reserved))
#define rgb_get(bmih,index,color)                                           \
            rgbquad_get((guint8 *)(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE),\
                                                                index,color)
#define rgb_set(bmih,index,color,value)                                     \
            rgbquad_set((guint8 *)(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE),\
                                                            index,color,value)
#define rgb_set_all(bmih,index,blue,green,red,reserved) (                   \
        rgbquad_set_all((guint8 *)(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE),\
                                        index,color,blue,green,red,reserved)


/******************************************************************************
*                                                                             *
* ja:ビットマップマクロ                                                       *
*                                                                             *
******************************************************************************/
/*  ja:RgbQuad構造体の数を求める
          width,幅
         height,高さ
      bit_count,1ピクセルあたりのビット数
    compression,圧縮形式
     color_used,色数
            RET,RgbQuad構造体の数                                           */
#define bx_color_used(width,height,bit_count,compression,color_used)        \
                ((color_used)>0?(color_used):((bit_count)<=8?1<<(bit_count):0))


/*  ja:RgbQuad構造体またはguint32配列のバイト数を求める
          width,幅
         height,高さ
      bit_count,1ピクセルあたりのビット数
    compression,圧縮形式
     color_used,色数
            RET,バイト数                                                    */
#define bx_color_bytes(width,height,bit_count,compression,color_used)       \
            ((compression)!=BI_COMP_BITFIELDS                               \
            ?bx_color_used(width,height,bit_count,compression,color_used)   \
                                            *RGBQUAD_SIZE:sizeof(guint32)*3)


/*  ja:BitmapInfoHeader構造体とRgbQuad構造体またはguint32配列のバイト数を求める
          width,幅
         height,高さ
      bit_count,1ピクセルあたりのビット数
    compression,圧縮形式
     color_used,色数
            RET,バイト数                                                    */
#define bx_header_bytes(width,height,bit_count,compression,color_used)      \
    (bx_color_bytes(width,height,bit_count,compression,color_used)+BMIH_SIZE)


/*  ja:1ラインのバイト数を求める
          width,幅
         height,高さ
      bit_count,1ピクセルあたりのビット数
    compression,圧縮形式
     color_used,色数
            RET,バイト数                                                    */
#define bx_line_bytes(width,height,bit_count,compression,color_used)        \
                                            ((((width)*(bit_count)+31)&~31)/8)


/*  ja:ピクセルデータのバイト数を求める
          width,幅
         height,高さ
      bit_count,1ピクセルあたりのビット数
    compression,圧縮形式
     color_used,色数
            RET,バイト数                                                    */
#define bx_image_bytes(width,height,bit_count,compression,color_used)       \
    (bx_line_bytes(width,height,bit_count,compression,color_used)*(height)) \


/*  ja:ビットマップのバイト数を求める
          width,幅
         height,高さ
      bit_count,1ピクセルあたりのビット数
    compression,圧縮形式
     color_used,色数
            RET,バイト数                                                    */
#define bx_all_bytes(width,height,bit_count,compression,color_used)         \
            (bx_header_bytes(width,height,bit_count,compression,color_used) \
            +bx_image_bytes(width,height,bit_count,compression,color_used))


/*  ja:RgbQuad構造体の数を求める
    bmih,BitmapInfoHeader構造体へのポインタ
     RET,RgbQuad構造体の数                                                  */
#define bm_color_used(bmih) bx_color_used(bmih_get_width(bmih),             \
                        bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                        bmih_get_compression(bmih),bmih_get_color_used(bmih))


/*  ja:RgbQuad構造体またはguint32配列のバイト数を求める
    bmih,BitmapInfoHeader構造体へのポインタ
     RET,バイト数                                                           */
#define bm_color_bytes(bmih) bx_color_bytes(bmih_get_width(bmih),           \
                        bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                        bmih_get_compression(bmih),bmih_get_color_used(bmih))


/*  ja:BitmapInfoHeader構造体とRgbQuad構造体またはguint32配列のバイト数を求める
    bmih,BitmapInfoHeader構造体へのポインタ
     RET,バイト数                                                           */
#define bm_header_bytes(bmih)                                               \
                    (bm_color_bytes(bmih)+MAX(bmih_get_size(bmih),BMIH_SIZE))


/*  ja:1ラインのバイト数を求める
    bmih,BitmapInfoHeader構造体へのポインタ
     RET,バイト数                                                           */
#define bm_line_bytes(bmih) bx_line_bytes(bmih_get_width(bmih),             \
                        bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                        bmih_get_compression(bmih),bmih_get_color_used(bmih))


/*  ja:ピクセルデータのバイト数を求める
    bmih,BitmapInfoHeader構造体へのポインタ
     RET,バイト数                                                           */
#define bm_image_bytes(bmih) bx_image_bytes(bmih_get_width(bmih),           \
                        bmih_get_height(bmih),bmih_get_bit_count(bmih),     \
                        bmih_get_compression(bmih),bmih_get_color_used(bmih))


/*  ja:ビットマップのバイト数を求める
    bmih,BitmapInfoHeader構造体へのポインタ
     RET,バイト数                                                           */
#define bm_all_bytes(bmih) (bm_header_bytes(bmih)+bm_image_bytes(bmih))


/******************************************************************************
*                                                                             *
* ja:ビットマップ加工関数                                                     *
*                                                                             *
******************************************************************************/
/*  ja:任意のパレットを求める
    count,求めるパレットの総数
      rgb,パレットを格納するバッファ
    table,パレット分布立方体                                                */
void
bitmap_make_palette (const gint   count,
                     RgbQuad     *rgb,
                     const guint  table[32][32][32]);


/*  ja:圧縮されたビットマップを復元する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_expand (const BitmapInfoHeader *bmih0,
               BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを1ビットビットマップに変換する(近似)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_approx_1 (const BitmapInfoHeader *bmih0,
                 BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを4ビットビットマップに変換する(近似)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_approx_4 (const BitmapInfoHeader *bmih0,
                 BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを8ビットビットマップに変換する(近似)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_approx_8 (const BitmapInfoHeader *bmih0,
                 BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを1ビットビットマップに変換する(誤差拡散)
    pbmih0,元のビットマップへのポインタ
    pbmih1,新しいビットマップへのポインタ                                   */
void
bitmap_diffuse_1 (const BitmapInfoHeader *bmih0,
                  BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを4ビットビットマップに変換する(誤差拡散)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_diffuse_4 (const BitmapInfoHeader *bmih0,
                  BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを8ビットビットマップに変換する(誤差拡散)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_diffuse_8 (const BitmapInfoHeader *bmih0,
                  BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを1ビットビットマップに変換する(識域)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_order_1 (const BitmapInfoHeader *bmih0,
                BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを4ビットビットマップに変換する(識域)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_order_4 (const BitmapInfoHeader *bmih0,
                BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを8ビットビットマップに変換する(識域)
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_order_8 (const BitmapInfoHeader *bmih0,
                BitmapInfoHeader       *bmih1);


/*  ja:フルカラー/32ビットビットマップを16ビットビットマップに変換する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_convert_16 (const BitmapInfoHeader *bmih0,
                   BitmapInfoHeader       *bmih1);


/*  ja:ビットマップをフルカラービットマップに変換する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_convert_24 (const BitmapInfoHeader *bmih0,
                   BitmapInfoHeader       *bmih1);


/*  ja:32ビットビットマップをフルカラービットマップに変換する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ
      rgb,背景色                                                            */
void
bitmap_convert_24_with_bgcolor (const BitmapInfoHeader *bmih0,
                                BitmapInfoHeader       *bmih1,
                                const RgbQuad          *rgb);


/*  ja:ビットマップを32ビットビットマップに変換する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_convert_32 (const BitmapInfoHeader *bmih0,
                   BitmapInfoHeader       *bmih1);


/*  ja:1ビットビットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom_1 (const BitmapInfoHeader *bmih0,
               BitmapInfoHeader       *bmih1);


/*  ja:4ビットビットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom_4 (const BitmapInfoHeader *bmih0,
               BitmapInfoHeader       *bmih1);


/*  ja:8ビットビットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom_8 (const BitmapInfoHeader *bmih0,
               BitmapInfoHeader       *bmih1);


/*  ja:16ビットビットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom_16 (const BitmapInfoHeader *bmih0,
                BitmapInfoHeader       *bmih1);


/*  ja:フルカラービットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom_24 (const BitmapInfoHeader *bmih0,
                BitmapInfoHeader       *bmih1);


/*  ja:32ビットビットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom_32 (const BitmapInfoHeader *bmih0,
                BitmapInfoHeader       *bmih1);


/*  ja:ビットマップのサイズを変更する
    bmih0,元のビットマップへのポインタ
    bmih1,新しいビットマップへのポインタ                                    */
void
bitmap_zoom (const BitmapInfoHeader *bmih0,
             BitmapInfoHeader       *bmih1);


/*  ja:32ビットビットマップを重ね合わせる
        bmih0,下になるビットマップへのポインタ
        bmih1,上になるビットマップへのポインタ
    x_overlay,X座標
    y_overlay,X座標
        trans,透過色,TRUE:有効,FALSE:無効                                   */
void
bitmap_overlay_32 (const BitmapInfoHeader *bmih0,
                   BitmapInfoHeader       *bmih1,
                   const gint              x_overlay,
                   const gint              y_overlay,
                   const gboolean          trans);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __BITMAP_H__ */
