/*
    Video maid
    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include "prop.h"


/******************************************************************************
*                                                                             *
* ja:プロパティダイアログ関数群                                               *
*                                                                             *
******************************************************************************/
#define MakeLangID(p,s) ((((guint16)(s))<<10)|(guint16)(p))
#define LANG_MAIN_NEUTRAL                   0x00
#define LANG_MAIN_AFRIKAANS                 0x36
#define LANG_MAIN_ALBANIAN                  0x1c
#define LANG_MAIN_ARABIC                    0x01
#define LANG_MAIN_ARMENIAN                  0x2b
#define LANG_MAIN_ASSAMESE                  0x4d
#define LANG_MAIN_AZERI                     0x2c
#define LANG_MAIN_BASQUE                    0x2d
#define LANG_MAIN_BENGALI                   0x45
#define LANG_MAIN_BULGARIAN                 0x02
#define LANG_MAIN_BYELORUSSIAN              0x23
#define LANG_MAIN_CATALAN                   0x03
#define LANG_MAIN_CHINESE                   0x04
#define LANG_MAIN_SERBO_CROATIAN            0x1a
#define LANG_MAIN_CROATIAN                  LANG_MAIN_SERBO_CROATIAN
#define LANG_MAIN_SERBIAN                   LANG_MAIN_SERBO_CROATIAN
#define LANG_MAIN_CZECH                     0x05
#define LANG_MAIN_DANISH                    0x06
#define LANG_MAIN_DUTCH                     0x13
#define LANG_MAIN_ENGLISH                   0x09
#define LANG_MAIN_ESTONIAN                  0x25
#define LANG_MAIN_FAEROESE                  0x38
#define LANG_MAIN_FARSI                     0x29
#define LANG_MAIN_FINNISH                   0x0b
#define LANG_MAIN_FRENCH                    0x0c
#define LANG_MAIN_GAELIC                    0x3c
#define LANG_MAIN_GEORGIAN                  0x37
#define LANG_MAIN_GERMAN                    0x07
#define LANG_MAIN_GREEK                     0x08
#define LANG_MAIN_GUJARATI                  0x47
#define LANG_MAIN_HEBREW                    0x0D
#define LANG_MAIN_HINDI                     0x39
#define LANG_MAIN_HUNGARIAN                 0x0e
#define LANG_MAIN_ICELANDIC                 0x0f
#define LANG_MAIN_INDONESIAN                0x21
#define LANG_MAIN_ITALIAN                   0x10
#define LANG_MAIN_JAPANESE                  0x11
#define LANG_MAIN_KANNADA                   0x4b
#define LANG_MAIN_KAZAKH                    0x3f
#define LANG_MAIN_KONKANI                   0x57
#define LANG_MAIN_KOREAN                    0x12
#define LANG_MAIN_LATVIAN                   0x26
#define LANG_MAIN_LITHUANIAN                0x27
#define LANG_MAIN_MACEDONIAN                0x2f
#define LANG_MAIN_MALAY                     0x3e
#define LANG_MAIN_MALAYALAM                 0x4c
#define LANG_MAIN_MALTESE                   0x3a
#define LANG_MAIN_MAORI                     0x28
#define LANG_MAIN_MARATHI                   0x4e
#define LANG_MAIN_NORWEGIAN                 0x14
#define LANG_MAIN_ORIYA                     0x48
#define LANG_MAIN_POLISH                    0x15
#define LANG_MAIN_PORTUGUESE                0x16
#define LANG_MAIN_PUNJABI                   0x46
#define LANG_MAIN_RHAETO_ROMANCE            0x17
#define LANG_MAIN_ROMANIAN                  0x18
#define LANG_MAIN_RUSSIAN                   0x19
#define LANG_MAIN_SAAMI                     0x3b
#define LANG_MAIN_SANSKRIT                  0x4f
#define LANG_MAIN_SLOVAK                    0x1b
#define LANG_MAIN_SLOVENIAN                 0x24
#define LANG_MAIN_SORBIAN                   0x2e
#define LANG_MAIN_SPANISH                   0x0a
#define LANG_MAIN_SUTU                      0x30
#define LANG_MAIN_SWAHILI                   0x41
#define LANG_MAIN_SWEDISH                   0x1d
#define LANG_MAIN_TAMIL                     0x49
#define LANG_MAIN_TATAR                     0x44
#define LANG_MAIN_TELUGU                    0x4a
#define LANG_MAIN_THAI                      0x1e
#define LANG_MAIN_TSONGA                    0x31
#define LANG_MAIN_TSWANA                    0x32
#define LANG_MAIN_TURKISH                   0x1f
#define LANG_MAIN_UKRAINIAN                 0x22
#define LANG_MAIN_URDU                      0x20
#define LANG_MAIN_UZBEK                     0x43
#define LANG_MAIN_VENDA                     0x33
#define LANG_MAIN_VIETNAMESE                0x2a
#define LANG_MAIN_XHOSA                     0x34
#define LANG_MAIN_ZULU                      0x35
#define LANG_MAIN_ESPERANTO                 0x8f
#define LANG_MAIN_WALON                     0x90
#define LANG_MAIN_CORNISH                   0x91
#define LANG_MAIN_WELSH                     0x92
#define LANG_MAIN_BRETON                    0x93
/* en:Sublanguage definitions */
#define LANG_SUB_NEUTRAL                    0x00    /* en:language neutral */
#define LANG_SUB_DEFAULT                    0x01    /* en:user default */
#define LANG_SUB_SYS_DEFAULT                0x02    /* en:system default */
#define LANG_SUB_ARABIC                     0x01
#define LANG_SUB_ARABIC_SAUDI_ARABIA        0x01
#define LANG_SUB_ARABIC_IRAQ                0x02
#define LANG_SUB_ARABIC_EGYPT               0x03
#define LANG_SUB_ARABIC_LIBYA               0x04
#define LANG_SUB_ARABIC_ALGERIA             0x05
#define LANG_SUB_ARABIC_MOROCCO             0x06
#define LANG_SUB_ARABIC_TUNISIA             0x07
#define LANG_SUB_ARABIC_OMAN                0x08
#define LANG_SUB_ARABIC_YEMEN               0x09
#define LANG_SUB_ARABIC_SYRIA               0x0a
#define LANG_SUB_ARABIC_JORDAN              0x0b
#define LANG_SUB_ARABIC_LEBANON             0x0c
#define LANG_SUB_ARABIC_KUWAIT              0x0d
#define LANG_SUB_ARABIC_UAE                 0x0e
#define LANG_SUB_ARABIC_BAHRAIN             0x0f
#define LANG_SUB_ARABIC_QATAR               0x10
#define LANG_SUB_CHINESE_TRADITIONAL        0x01
#define LANG_SUB_CHINESE_SIMPLIFIED         0x02
#define LANG_SUB_CHINESE_HONGKONG           0x03
#define LANG_SUB_CHINESE_SINGAPORE          0x04
#define LANG_SUB_CHINESE_MACAU              0x05
#define LANG_SUB_DUTCH                      0x01
#define LANG_SUB_DUTCH_BELGIAN              0x02
#define LANG_SUB_DUTCH_SURINAM              0x03
#define LANG_SUB_ENGLISH_US                 0x01
#define LANG_SUB_ENGLISH_UK                 0x02
#define LANG_SUB_ENGLISH_AUS                0x03
#define LANG_SUB_ENGLISH_CAN                0x04
#define LANG_SUB_ENGLISH_NZ                 0x05
#define LANG_SUB_ENGLISH_EIRE               0x06
#define LANG_SUB_ENGLISH_SAFRICA            0x07
#define LANG_SUB_ENGLISH_JAMAICA            0x08
#define LANG_SUB_ENGLISH_CARRIBEAN          0x09
#define LANG_SUB_ENGLISH_BELIZE             0x0a
#define LANG_SUB_ENGLISH_TRINIDAD           0x0b
#define LANG_SUB_ENGLISH_ZIMBABWE           0x0c
#define LANG_SUB_ENGLISH_PHILIPPINES        0x0d
#define LANG_SUB_FRENCH                     0x01
#define LANG_SUB_FRENCH_BELGIAN             0x02
#define LANG_SUB_FRENCH_CANADIAN            0x03
#define LANG_SUB_FRENCH_SWISS               0x04
#define LANG_SUB_FRENCH_LUXEMBOURG          0x05
#define LANG_SUB_FRENCH_MONACO              0x06
#define LANG_SUB_GERMAN                     0x01
#define LANG_SUB_GERMAN_SWISS               0x02
#define LANG_SUB_GERMAN_AUSTRIAN            0x03
#define LANG_SUB_GERMAN_LUXEMBOURG          0x04
#define LANG_SUB_GERMAN_LIECHTENSTEIN       0x05
#define LANG_SUB_ITALIAN                    0x01
#define LANG_SUB_ITALIAN_SWISS              0x02
#define LANG_SUB_KOREAN                     0x01
#define LANG_SUB_KOREAN_JOHAB               0x02
#define LANG_SUB_NORWEGIAN_BOKMAL           0x01
#define LANG_SUB_NORWEGIAN_NYNORSK          0x02
#define LANG_SUB_PORTUGUESE                 0x02
#define LANG_SUB_PORTUGUESE_BRAZILIAN       0x01
#define LANG_SUB_SPANISH                    0x01
#define LANG_SUB_SPANISH_MEXICAN            0x02
#define LANG_SUB_SPANISH_MODERN             0x03
#define LANG_SUB_SPANISH_GUATEMALA          0x04
#define LANG_SUB_SPANISH_COSTARICA          0x05
#define LANG_SUB_SPANISH_PANAMA             0x06
#define LANG_SUB_SPANISH_DOMINICAN          0x07
#define LANG_SUB_SPANISH_VENEZUELA          0x08
#define LANG_SUB_SPANISH_COLOMBIA           0x09
#define LANG_SUB_SPANISH_PERU               0x0a
#define LANG_SUB_SPANISH_ARGENTINA          0x0b
#define LANG_SUB_SPANISH_ECUADOR            0x0c
#define LANG_SUB_SPANISH_CHILE              0x0d
#define LANG_SUB_SPANISH_URUGUAY            0x0e
#define LANG_SUB_SPANISH_PARAGUAY           0x0f
#define LANG_SUB_SPANISH_BOLIVIA            0x10
#define LANG_SUB_SPANISH_EL_SALVADOR        0x11
#define LANG_SUB_SPANISH_HONDURAS           0x12
#define LANG_SUB_SPANISH_NICARAGUA          0x13
#define LANG_SUB_SPANISH_PUERTO_RICO        0x14
#define LANG_SUB_ROMANIAN                   0x01
#define LANG_SUB_ROMANIAN_MOLDAVIA          0x02
#define LANG_SUB_RUSSIAN                    0x01
#define LANG_SUB_RUSSIAN_MOLDAVIA           0x02
#define LANG_SUB_CROATIAN                   0x01
#define LANG_SUB_SERBIAN                    0x02
#define LANG_SUB_SERBIAN_LATIN              0x03
#define LANG_SUB_SWEDISH                    0x01
#define LANG_SUB_SWEDISH_FINLAND            0x02
#define LANG_SUB_LITHUANIAN                 0x01
#define LANG_SUB_LITHUANIAN_CLASSIC         0x02
#define LANG_SUB_AZERI                      0x01
#define LANG_SUB_AZERI_CYRILLIC             0x02
#define LANG_SUB_GAELIC                     0x01
#define LANG_SUB_GAELIC_SCOTTISH            0x02
#define LANG_SUB_GAELIC_MANX                0x03
#define LANG_SUB_MALAY                      0x01
#define LANG_SUB_MALAY_BRUNEI_DARUSSALAM    0x02
#define LANG_SUB_UZBEK                      0x01
#define LANG_SUB_UZBEK_CYRILLIC             0x02
#define LANG_SUB_URDU_PAKISTAN              0x01


typedef struct _PropertyDialog
{
  gboolean ok_sensitive, changed_entry, video, audio;
  GtkWidget *dialog, *combo0, *combo1;
  GtkWidget *entry, *spin0, *spin1, *spin2;
} PropertyDialog;


static struct
{
  const gchar *text;
  const guint16 id;
} lang[] = {
{N_("Neutral"),                    MakeLangID (LANG_MAIN_NEUTRAL,        LANG_SUB_NEUTRAL)},
{N_("User Default"),               MakeLangID (LANG_MAIN_NEUTRAL,        LANG_SUB_DEFAULT)},
{N_("System Default"),             MakeLangID (LANG_MAIN_NEUTRAL,        LANG_SUB_SYS_DEFAULT)},
{N_("Afrikaans"),                  MakeLangID (LANG_MAIN_AFRIKAANS,      LANG_SUB_DEFAULT)},
{N_("Albanian"),                   MakeLangID (LANG_MAIN_ALBANIAN,       LANG_SUB_DEFAULT)},
{N_("Arabic"),                     MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC)},
{N_("Arabic (Saudi Arabia)"),      MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_SAUDI_ARABIA)},
{N_("Arabic (Iraq)"),              MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_IRAQ)},
{N_("Arabic (Egypt)"),             MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_EGYPT)},
{N_("Arabic (Libya)"),             MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_LIBYA)},
{N_("Arabic (Algeria)"),           MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_ALGERIA)},
{N_("Arabic (Morocco)"),           MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_MOROCCO)},
{N_("Arabic (Tunisia)"),           MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_TUNISIA)},
{N_("Arabic (Oman)"),              MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_OMAN)},
{N_("Arabic (Yemen)"),             MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_YEMEN)},
{N_("Arabic (Syria)"),             MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_SYRIA)},
{N_("Arabic (Jordan)"),            MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_JORDAN)},
{N_("Arabic (Lebanon)"),           MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_LEBANON)},
{N_("Arabic (Kuwait)"),            MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_KUWAIT)},
{N_("Arabic (UAE)"),               MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_UAE)},
{N_("Arabic (Bahrain)"),           MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_BAHRAIN)},
{N_("Arabic (Qatar)"),             MakeLangID (LANG_MAIN_ARABIC,         LANG_SUB_ARABIC_QATAR)},
{N_("Armenian"),                   MakeLangID (LANG_MAIN_ARMENIAN,       LANG_SUB_DEFAULT)},
{N_("Assamese"),                   MakeLangID (LANG_MAIN_ASSAMESE,       LANG_SUB_DEFAULT)},
{N_("Azeri"),                      MakeLangID (LANG_MAIN_AZERI,          LANG_SUB_AZERI)},
{N_("Azeri (Cyrillic)"),           MakeLangID (LANG_MAIN_AZERI,          LANG_SUB_AZERI_CYRILLIC)},
{N_("Basque"),                     MakeLangID (LANG_MAIN_BASQUE,         LANG_SUB_DEFAULT)},
{N_("Bengali"),                    MakeLangID (LANG_MAIN_BENGALI,        LANG_SUB_DEFAULT)},
{N_("Bulgarian"),                  MakeLangID (LANG_MAIN_BULGARIAN,      LANG_SUB_DEFAULT)},
{N_("Byelorussian"),               MakeLangID (LANG_MAIN_BYELORUSSIAN,   LANG_SUB_DEFAULT)},
{N_("Catalan"),                    MakeLangID (LANG_MAIN_CATALAN,        LANG_SUB_DEFAULT)},
{N_("Chinese (Traditional)"),      MakeLangID (LANG_MAIN_CHINESE,        LANG_SUB_CHINESE_TRADITIONAL)},
{N_("Chinese (Simplified)"),       MakeLangID (LANG_MAIN_CHINESE,        LANG_SUB_CHINESE_SIMPLIFIED)},
{N_("Chinese (HongKong)"),         MakeLangID (LANG_MAIN_CHINESE,        LANG_SUB_CHINESE_HONGKONG)},
{N_("Chinese (Singapore)"),        MakeLangID (LANG_MAIN_CHINESE,        LANG_SUB_CHINESE_SINGAPORE)},
{N_("Chinese (Macau)"),            MakeLangID (LANG_MAIN_CHINESE,        LANG_SUB_CHINESE_MACAU)},
{N_("Croatian"),                   MakeLangID (LANG_MAIN_SERBO_CROATIAN, LANG_SUB_CROATIAN)},
{N_("Serbian"),                    MakeLangID (LANG_MAIN_SERBO_CROATIAN, LANG_SUB_SERBIAN)},
{N_("Serbian (Latin)"),            MakeLangID (LANG_MAIN_SERBO_CROATIAN, LANG_SUB_SERBIAN_LATIN)},
{N_("Czech"),                      MakeLangID (LANG_MAIN_CZECH,          LANG_SUB_DEFAULT)},
{N_("Danish"),                     MakeLangID (LANG_MAIN_DANISH,         LANG_SUB_DEFAULT)},
{N_("Dutch"),                      MakeLangID (LANG_MAIN_DUTCH,          LANG_SUB_DUTCH)},
{N_("Dutch (Belgian)"),            MakeLangID (LANG_MAIN_DUTCH,          LANG_SUB_DUTCH_BELGIAN)},
{N_("Dutch (Surinam)"),            MakeLangID (LANG_MAIN_DUTCH,          LANG_SUB_DUTCH_SURINAM)},
{N_("English (US)"),               MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_US)},
{N_("English (UK)"),               MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_UK)},
{N_("English (Australia)"),        MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_AUS)},
{N_("English (Canada)"),           MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_CAN)},
{N_("English (New Zealand)"),      MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_NZ)},
{N_("English (Ireland)"),          MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_EIRE)},
{N_("English (South Africa)"),     MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_SAFRICA)},
{N_("English (Jamaica)"),          MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_JAMAICA)},
{N_("English (Carribean)"),        MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_CARRIBEAN)},
{N_("English (Belize)"),           MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_BELIZE)},
{N_("English (Trinidad)"),         MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_TRINIDAD)},
{N_("English (Zimbabwe)"),         MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_ZIMBABWE)},
{N_("English (Philippines)"),      MakeLangID (LANG_MAIN_ENGLISH,        LANG_SUB_ENGLISH_PHILIPPINES)},
{N_("Estonian"),                   MakeLangID (LANG_MAIN_ESTONIAN,       LANG_SUB_DEFAULT)},
{N_("Faeroese"),                   MakeLangID (LANG_MAIN_FAEROESE,       LANG_SUB_DEFAULT)},
{N_("Farsi"),                      MakeLangID (LANG_MAIN_FARSI,          LANG_SUB_DEFAULT)},
{N_("Finnish"),                    MakeLangID (LANG_MAIN_FINNISH,        LANG_SUB_DEFAULT)},
{N_("French"),                     MakeLangID (LANG_MAIN_FRENCH,         LANG_SUB_FRENCH)},
{N_("French (Belgian)"),           MakeLangID (LANG_MAIN_FRENCH,         LANG_SUB_FRENCH_BELGIAN)},
{N_("French (Canadian)"),          MakeLangID (LANG_MAIN_FRENCH,         LANG_SUB_FRENCH_CANADIAN)},
{N_("French (Swiss)"),             MakeLangID (LANG_MAIN_FRENCH,         LANG_SUB_FRENCH_SWISS)},
{N_("French (Luxembourg)"),        MakeLangID (LANG_MAIN_FRENCH,         LANG_SUB_FRENCH_LUXEMBOURG)},
{N_("French (Monaco)"),            MakeLangID (LANG_MAIN_FRENCH,         LANG_SUB_FRENCH_MONACO)},
{N_("Gaelic"),                     MakeLangID (LANG_MAIN_GAELIC,         LANG_SUB_GAELIC)},
{N_("Gaelic (Scottish)"),          MakeLangID (LANG_MAIN_GAELIC,         LANG_SUB_GAELIC_SCOTTISH)},
{N_("Gaelic (Manx)"),              MakeLangID (LANG_MAIN_GAELIC,         LANG_SUB_GAELIC_MANX)},
{N_("Georgian"),                   MakeLangID (LANG_MAIN_GEORGIAN,       LANG_SUB_DEFAULT)},
{N_("German"),                     MakeLangID (LANG_MAIN_GERMAN,         LANG_SUB_GERMAN)},
{N_("German (Swiss)"),             MakeLangID (LANG_MAIN_GERMAN,         LANG_SUB_GERMAN_SWISS)},
{N_("German (Austrian)"),          MakeLangID (LANG_MAIN_GERMAN,         LANG_SUB_GERMAN_AUSTRIAN)},
{N_("German (Luxembourg)"),        MakeLangID (LANG_MAIN_GERMAN,         LANG_SUB_GERMAN_LUXEMBOURG)},
{N_("German (Liechtenstein)"),     MakeLangID (LANG_MAIN_GERMAN,         LANG_SUB_GERMAN_LIECHTENSTEIN)},
{N_("Greek"),                      MakeLangID (LANG_MAIN_GREEK,          LANG_SUB_DEFAULT)},
{N_("Gujarati"),                   MakeLangID (LANG_MAIN_GUJARATI,       LANG_SUB_DEFAULT)},
{N_("Hebrew"),                     MakeLangID (LANG_MAIN_HEBREW,         LANG_SUB_DEFAULT)},
{N_("Hindi"),                      MakeLangID (LANG_MAIN_HINDI,          LANG_SUB_DEFAULT)},
{N_("Hungarian"),                  MakeLangID (LANG_MAIN_HUNGARIAN,      LANG_SUB_DEFAULT)},
{N_("Icelandic"),                  MakeLangID (LANG_MAIN_ICELANDIC,      LANG_SUB_DEFAULT)},
{N_("Indonesian"),                 MakeLangID (LANG_MAIN_INDONESIAN,     LANG_SUB_DEFAULT)},
{N_("Italian"),                    MakeLangID (LANG_MAIN_ITALIAN,        LANG_SUB_ITALIAN)},
{N_("Italian (Swiss)"),            MakeLangID (LANG_MAIN_ITALIAN,        LANG_SUB_ITALIAN_SWISS)},
{N_("Japanese"),                   MakeLangID (LANG_MAIN_JAPANESE,       LANG_SUB_DEFAULT)},
{N_("Kannada"),                    MakeLangID (LANG_MAIN_KANNADA,        LANG_SUB_DEFAULT)},
{N_("Kazakh"),                     MakeLangID (LANG_MAIN_KAZAKH,         LANG_SUB_DEFAULT)},
{N_("Konkani"),                    MakeLangID (LANG_MAIN_KONKANI,        LANG_SUB_DEFAULT)},
{N_("Korean"),                     MakeLangID (LANG_MAIN_KOREAN,         LANG_SUB_KOREAN)},
{N_("Korean (Johab)"),             MakeLangID (LANG_MAIN_KOREAN,         LANG_SUB_KOREAN_JOHAB)},
{N_("Latvian"),                    MakeLangID (LANG_MAIN_LATVIAN,        LANG_SUB_DEFAULT)},
{N_("Lithuanian"),                 MakeLangID (LANG_MAIN_LITHUANIAN,     LANG_SUB_LITHUANIAN)},
{N_("Lithuanian (Classic)"),       MakeLangID (LANG_MAIN_LITHUANIAN,     LANG_SUB_LITHUANIAN_CLASSIC)},
{N_("Macedonian"),                 MakeLangID (LANG_MAIN_MACEDONIAN,     LANG_SUB_DEFAULT)},
{N_("Malay"),                      MakeLangID (LANG_MAIN_MALAY,          LANG_SUB_MALAY)},
{N_("Malay (Brunbei Darussalam)"), MakeLangID (LANG_MAIN_MALAY,          LANG_SUB_MALAY_BRUNEI_DARUSSALAM)},
{N_("Malayalam"),                  MakeLangID (LANG_MAIN_MALAYALAM,      LANG_SUB_DEFAULT)},
{N_("Maltese"),                    MakeLangID (LANG_MAIN_MALTESE,        LANG_SUB_DEFAULT)},
{N_("Maori"),                      MakeLangID (LANG_MAIN_MAORI,          LANG_SUB_DEFAULT)},
{N_("Marathi"),                    MakeLangID (LANG_MAIN_MARATHI,        LANG_SUB_DEFAULT)},
{N_("Norwegian"),                  MakeLangID (LANG_MAIN_NORWEGIAN,      LANG_SUB_NORWEGIAN_BOKMAL)},
{N_("Norwegian (Nynorsk)"),        MakeLangID (LANG_MAIN_NORWEGIAN,      LANG_SUB_NORWEGIAN_NYNORSK)},
{N_("Oriya"),                      MakeLangID (LANG_MAIN_ORIYA,          LANG_SUB_DEFAULT)},
{N_("Polish"),                     MakeLangID (LANG_MAIN_POLISH,         LANG_SUB_DEFAULT)},
{N_("Portuguese"),                 MakeLangID (LANG_MAIN_PORTUGUESE,     LANG_SUB_PORTUGUESE)},
{N_("Portuguese (Brazilian)"),     MakeLangID (LANG_MAIN_PORTUGUESE,     LANG_SUB_PORTUGUESE_BRAZILIAN)},
{N_("Punjabi"),                    MakeLangID (LANG_MAIN_PUNJABI,        LANG_SUB_DEFAULT)},
{N_("Rhaeto Romance"),             MakeLangID (LANG_MAIN_RHAETO_ROMANCE, LANG_SUB_DEFAULT)},
{N_("Romanian"),                   MakeLangID (LANG_MAIN_ROMANIAN,       LANG_SUB_ROMANIAN)},
{N_("Romanian (Moldavia)"),        MakeLangID (LANG_MAIN_ROMANIAN,       LANG_SUB_ROMANIAN_MOLDAVIA)},
{N_("Russian"),                    MakeLangID (LANG_MAIN_RUSSIAN,        LANG_SUB_RUSSIAN)},
{N_("Russian (Moldavia)"),         MakeLangID (LANG_MAIN_RUSSIAN,        LANG_SUB_RUSSIAN_MOLDAVIA)},
{N_("Saami"),                      MakeLangID (LANG_MAIN_SAAMI,          LANG_SUB_DEFAULT)},
{N_("Sanskrit"),                   MakeLangID (LANG_MAIN_SANSKRIT,       LANG_SUB_DEFAULT)},
{N_("Slovak"),                     MakeLangID (LANG_MAIN_SLOVAK,         LANG_SUB_DEFAULT)},
{N_("Slovenian"),                  MakeLangID (LANG_MAIN_SLOVENIAN,      LANG_SUB_DEFAULT)},
{N_("Sorbian"),                    MakeLangID (LANG_MAIN_SORBIAN,        LANG_SUB_DEFAULT)},
{N_("Spanish"),                    MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH)},
{N_("Spanish (Mexican)"),          MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_MEXICAN)},
{N_("Spanish (Modern)"),           MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_MODERN)},
{N_("Spanish (Guatemala)"),        MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_GUATEMALA)},
{N_("Spanish (Costarica)"),        MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_COSTARICA)},
{N_("Spanish (Panama)"),           MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_PANAMA)},
{N_("Spanish (Dominican)"),        MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_DOMINICAN)},
{N_("Spanish (Venezuela)"),        MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_VENEZUELA)},
{N_("Spanish (Colombia)"),         MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_COLOMBIA)},
{N_("Spanish (Peru)"),             MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_PERU)},
{N_("Spanish (Argentina)"),        MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_ARGENTINA)},
{N_("Spanish (Ecuador)"),          MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_ECUADOR)},
{N_("Spanish (Chile)"),            MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_CHILE)},
{N_("Spanish (Uruguay)"),          MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_URUGUAY)},
{N_("Spanish (Paraguay)"),         MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_PARAGUAY)},
{N_("Spanish (Bolivia)"),          MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_BOLIVIA)},
{N_("Spanish (El Salvador)"),      MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_EL_SALVADOR)},
{N_("Spanish (Honduras)"),         MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_HONDURAS)},
{N_("Spanish (Nicaragua)"),        MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_NICARAGUA)},
{N_("Spanish (Puerto Rico)"),      MakeLangID (LANG_MAIN_SPANISH,        LANG_SUB_SPANISH_PUERTO_RICO)},
{N_("Sutu"),                       MakeLangID (LANG_MAIN_SUTU,           LANG_SUB_DEFAULT)},
{N_("Swahili"),                    MakeLangID (LANG_MAIN_SWAHILI,        LANG_SUB_DEFAULT)},
{N_("Swedish"),                    MakeLangID (LANG_MAIN_SWEDISH,        LANG_SUB_SWEDISH)},
{N_("Swedish (Finland)"),          MakeLangID (LANG_MAIN_SWEDISH,        LANG_SUB_SWEDISH_FINLAND)},
{N_("Tamil"),                      MakeLangID (LANG_MAIN_TAMIL,          LANG_SUB_DEFAULT)},
{N_("Tatar"),                      MakeLangID (LANG_MAIN_TATAR,          LANG_SUB_DEFAULT)},
{N_("Telugu"),                     MakeLangID (LANG_MAIN_TELUGU,         LANG_SUB_DEFAULT)},
{N_("Thai"),                       MakeLangID (LANG_MAIN_THAI,           LANG_SUB_DEFAULT)},
{N_("Tsonga"),                     MakeLangID (LANG_MAIN_TSONGA,         LANG_SUB_DEFAULT)},
{N_("Tswana"),                     MakeLangID (LANG_MAIN_TSWANA,         LANG_SUB_DEFAULT)},
{N_("Turkish"),                    MakeLangID (LANG_MAIN_TURKISH,        LANG_SUB_DEFAULT)},
{N_("Ukrainian"),                  MakeLangID (LANG_MAIN_UKRAINIAN,      LANG_SUB_DEFAULT)},
{N_("Urdu"),                       MakeLangID (LANG_MAIN_URDU,           LANG_SUB_URDU_PAKISTAN)},
{N_("Uzbek"),                      MakeLangID (LANG_MAIN_UZBEK,          LANG_SUB_UZBEK)},
{N_("Uzbek (Cyrillic)"),           MakeLangID (LANG_MAIN_UZBEK,          LANG_SUB_UZBEK_CYRILLIC)},
{N_("Venda"),                      MakeLangID (LANG_MAIN_VENDA,          LANG_SUB_DEFAULT)},
{N_("Vietnamese"),                 MakeLangID (LANG_MAIN_VIETNAMESE,     LANG_SUB_DEFAULT)},
{N_("Xhosa"),                      MakeLangID (LANG_MAIN_XHOSA,          LANG_SUB_DEFAULT)},
{N_("Zulu"),                       MakeLangID (LANG_MAIN_ZULU,           LANG_SUB_DEFAULT)},
{N_("Esperanto"),                  MakeLangID (LANG_MAIN_ESPERANTO,      LANG_SUB_DEFAULT)},
{N_("Walon"),                      MakeLangID (LANG_MAIN_WALON,          LANG_SUB_DEFAULT)},
{N_("Cornish"),                    MakeLangID (LANG_MAIN_CORNISH,        LANG_SUB_DEFAULT)},
{N_("Welsh"),                      MakeLangID (LANG_MAIN_WELSH,          LANG_SUB_DEFAULT)},
{N_("Breton"),                     MakeLangID (LANG_MAIN_BRETON,         LANG_SUB_DEFAULT)},
{NULL,                             0}};


/* ja:エントリー/スピンボタンでリターンが押された */
static void
prop_dialog_activate (GtkWidget      *widget,
                      PropertyDialog *propdlg)
{
  if (propdlg->ok_sensitive)
    gtk_dialog_response (GTK_DIALOG (propdlg->dialog), GTK_RESPONSE_OK);
}


/* ja:ラジオボタンが変更された */
#define prop_dialog_toggled prop_dialog_changed
/* ja:コンボボックス/エントリー/スピンボタンが変更された */
static void
prop_dialog_changed (GtkWidget      *widget,
                     PropertyDialog *propdlg)
{
  gboolean result = TRUE;
  const gchar *text;
  gint i, rate, scale, value;

  if (propdlg->video)
    {
      text = gtk_entry_get_text
#if GTK_CHECK_VERSION(2,4,0)
                (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg->combo0))));
#else /* not GTK_CHECK_VERSION(2,4,0) */
                            (GTK_ENTRY (GTK_COMBO (propdlg->combo0)->entry));
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      for (i = 0; lang[i].text; i++)
        if (g_strcmp (_(lang[i].text), text) == 0)
          break;
      if (!lang[i].text && !misc_str_to_val (&value, text, 16, FALSE))
        result = FALSE;
    }
  if (result && propdlg->audio)
    {
      text = gtk_entry_get_text
#if GTK_CHECK_VERSION(2,4,0)
                (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg->combo1))));
#else /* not GTK_CHECK_VERSION(2,4,0) */
                            (GTK_ENTRY (GTK_COMBO (propdlg->combo1)->entry));
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      for (i = 0; lang[i].text; i++)
        if (g_strcmp (_(lang[i].text), text) == 0)
          break;
      if (!lang[i].text && !misc_str_to_val (&value, text, 16, FALSE))
        result = FALSE;
    }
  propdlg->ok_sensitive = result
        && misc_str_to_float (&rate, &scale,
                            gtk_entry_get_text (GTK_ENTRY (propdlg->entry)))
        && rate > 0
        && (!propdlg->video || (gtk_spin_button_get_value_as_int
                                        (GTK_SPIN_BUTTON (propdlg->spin0)) > 0
                && gtk_spin_button_get_value_as_int
                                    (GTK_SPIN_BUTTON (propdlg->spin1)) > 0))
        && (!propdlg->audio || gtk_spin_button_get_value_as_int
                                    (GTK_SPIN_BUTTON (propdlg->spin2)) > 0);
  gtk_dialog_set_response_sensitive (GTK_DIALOG (propdlg->dialog),
                                    GTK_RESPONSE_OK, propdlg->ok_sensitive);
}


/* ja:エントリーが変更された */
static void
prop_dialog_changed_entry (GtkWidget      *widget,
                           PropertyDialog *propdlg)
{
  propdlg->changed_entry = TRUE;
  prop_dialog_changed (widget, propdlg);
}


/*  ja:プロパティダイアログ表示
    propcfg,プロパティ構造体
        RET,TRUE:OK,FALSE:Cancel                                            */
gboolean
property_dialog (PropertyConfig *propcfg)
{
  gboolean result;
  gchar *text, *title;
  gint i;
  GList *glist;
  GtkStockItem stock_item;
  GtkWidget *combo, *entry0, *entry1, *radio0, *radio1;
  GtkWidget *spin0, *spin1, *spin2, *spin3;
  GtkWidget *label0, *label1, *label2, *label3, *label4;
  GtkWidget *label5, *label6, *label7, *label8;
  GtkWidget *frame0, *frame1, *hbox, *vbox;
  PropertyDialog propdlg;
  static struct
  {
    const gchar *text;
    const gint bit;
  } bit_count[] = {{N_("Monochrome"),  1}, {N_("16 colors"),  4},
                   {N_("256 colors"),  8}, {N_("16 bits"),   16},
                   {N_("Full Color"), 24}, {N_("32 bits"),   32}, {NULL, 0}};

  /* ja:メインウインドウ */
  gtk_stock_lookup (GTK_STOCK_PROPERTIES, &stock_item);
  title = misc_mnemonic_to_text (stock_item.label);
  propdlg.dialog = gtk_dialog_new_with_buttons
                            (title,
                            GTK_WINDOW (window),
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                            GTK_STOCK_OK, GTK_RESPONSE_OK,
                            NULL);
  g_free (title);
  g_signal_connect (G_OBJECT (propdlg.dialog), "key-press-event",
                                    G_CALLBACK (misc_dialog_key_press), NULL);
  /* ja:エントリー */
  propdlg.entry = gtk_entry_new ();
  text = misc_str_from_float (propcfg->rate, propcfg->scale);
  gtk_entry_set_text (GTK_ENTRY (propdlg.entry), text);
  g_free (text);
  g_signal_connect (G_OBJECT (propdlg.entry), "activate",
                            G_CALLBACK (prop_dialog_activate), &propdlg);
  g_signal_connect (G_OBJECT (propdlg.entry), "changed",
                            G_CALLBACK (prop_dialog_changed_entry), &propdlg);
  gtk_editable_select_region (GTK_EDITABLE (propdlg.entry), 0, -1);
  /* ja:ラベル */
  label0 = gtk_label_new_with_mnemonic  (_("_Frame per second"));
  gtk_label_set_mnemonic_widget (GTK_LABEL (label0), propdlg.entry);

  if (propcfg->width > 0 && propcfg->height > 0 && propcfg->bit_count != 0)
    {
      /* ja:コンボボックス */
      glist = NULL;
      for (i = 3; lang[i].text; i++)
        glist = g_list_insert_sorted (glist, (gpointer)_(lang[i].text),
                                                        (GCompareFunc)strcmp);
      for (i = 2; i >= 0; i--)
        glist = g_list_prepend (glist, (gpointer)_(lang[i].text));
#if GTK_CHECK_VERSION(2,4,0)
      propdlg.combo0 = gtk_combo_box_entry_new_text ();
      for (i = g_list_length (glist) - 1; i >= 0; i--)
        gtk_combo_box_prepend_text (GTK_COMBO_BOX (propdlg.combo0),
                                                g_list_nth_data (glist, i));
#else /* not GTK_CHECK_VERSION(2,4,0) */
      propdlg.combo0 = gtk_combo_new ();
      gtk_combo_set_popdown_strings (GTK_COMBO (propdlg.combo0), glist);
      gtk_combo_set_value_in_list (GTK_COMBO (propdlg.combo0), FALSE, FALSE);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      for (i = 0; lang[i].text; i++)
        if (lang[i].id == propcfg->language0)
          {
#if GTK_CHECK_VERSION(2,4,0)
            gtk_entry_set_text
                    (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg.combo0))),
                                                            _(lang[i].text));
#else /* not GTK_CHECK_VERSION(2,4,0) */
            gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (propdlg.combo0)->entry),
                                                            _(lang[i].text));
#endif /* not GTK_CHECK_VERSION(2,4,0) */
            break;
          }
      if (!lang[i].text)
        {
          text = g_strdup_printf ("%X", propcfg->language0);
#if GTK_CHECK_VERSION(2,4,0)
          gtk_entry_set_text
            (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg.combo0))), text);
#else /* not GTK_CHECK_VERSION(2,4,0) */
          gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (propdlg.combo0)->entry),
                                                                        text);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
          g_free (text);
        }
#if GTK_CHECK_VERSION(2,4,0)
      g_signal_connect
                    (G_OBJECT (gtk_bin_get_child (GTK_BIN (propdlg.combo0))),
                        "changed", G_CALLBACK (prop_dialog_changed), &propdlg);
#else /* not GTK_CHECK_VERSION(2,4,0) */
      g_signal_connect (G_OBJECT (GTK_COMBO (propdlg.combo0)->entry),
                        "changed", G_CALLBACK (prop_dialog_changed), &propdlg);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      g_list_free (glist);
      /* ja:コンボボックス */
#if GTK_CHECK_VERSION(2,4,0)
      combo = gtk_combo_box_new_text ();
      for (i = 0; bit_count[i].text; i++)
        {
          gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
                                                        _(bit_count[i].text));
          if (bit_count[i].bit == propcfg->bit_count)
            gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
        }
      g_signal_connect (G_OBJECT (combo), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
#else /* not GTK_CHECK_VERSION(2,4,0) */
      glist = NULL;
      for (i = 0; bit_count[i].text; i++)
        glist = g_list_append (glist, (gpointer)_(bit_count[i].text));
      combo = gtk_combo_new ();
      gtk_combo_set_popdown_strings (GTK_COMBO (combo), glist);
      gtk_combo_set_value_in_list (GTK_COMBO (combo), TRUE, FALSE);
      gtk_editable_set_editable
                            (GTK_EDITABLE (GTK_COMBO (combo)->entry), FALSE);
      for (i = 0; bit_count[i].text; i++)
        if (bit_count[i].bit == propcfg->bit_count)
          gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry),
                                                        _(bit_count[i].text));
      g_signal_connect (G_OBJECT (GTK_COMBO (combo)->entry), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      /* ja:エントリー */
      entry0 = gtk_entry_new ();
      if (propcfg->name0)
        gtk_entry_set_text (GTK_ENTRY (entry0), propcfg->name0);
      g_signal_connect (G_OBJECT (entry0), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (entry0), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      /* ja:スピンボタン */
      spin0 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                        (propcfg->priority0, 0, G_MAXSHORT, 1, 5, 0)), 0, 0);
      propdlg.spin0 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                                (propcfg->width, 1, G_MAXINT, 1, 5, 0)), 0, 0);
      propdlg.spin1 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                            (propcfg->height, 1, G_MAXINT, 1, 5, 0)), 0, 0);
      spin1 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                (propcfg->y_pixels_per_meter, 0, G_MAXINT, 1, 5, 0)), 0, 0);
      spin2 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                (propcfg->y_pixels_per_meter, 0, G_MAXINT, 1, 5, 0)), 0, 0);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin0), TRUE);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (propdlg.spin0), TRUE);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (propdlg.spin1), TRUE);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin1), TRUE);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin2), TRUE);
      g_signal_connect (G_OBJECT (spin0), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (propdlg.spin0), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (propdlg.spin1), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (spin1), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (spin2), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (spin0), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      g_signal_connect (G_OBJECT (propdlg.spin0), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      g_signal_connect (G_OBJECT (propdlg.spin1), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      g_signal_connect (G_OBJECT (spin1), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      g_signal_connect (G_OBJECT (spin2), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      /* ja:ラベル */
      label1 = gtk_label_new_with_mnemonic (_("_Name"));
      label2 = gtk_label_new_with_mnemonic (_("_Priority"));
      label3 = gtk_label_new_with_mnemonic (_("_Language"));
      label4 = gtk_label_new_with_mnemonic (_("_Width"));
      label5 = gtk_label_new_with_mnemonic (_("_Height"));
      label6 = gtk_label_new_with_mnemonic (_("_Color"));
      label7 = gtk_label_new_with_mnemonic (_("_X pixels per meter"));
      label8 = gtk_label_new_with_mnemonic (_("_Y pixels per meter"));
      gtk_label_set_mnemonic_widget (GTK_LABEL (label1), entry0);
      gtk_label_set_mnemonic_widget (GTK_LABEL (label2), spin0);
#if GTK_CHECK_VERSION(2,4,0)
      gtk_label_set_mnemonic_widget (GTK_LABEL (label3),
                                gtk_bin_get_child (GTK_BIN (propdlg.combo0)));
#else /* not GTK_CHECK_VERSION(2,4,0) */
      gtk_label_set_mnemonic_widget (GTK_LABEL (label3),
                                            GTK_COMBO (propdlg.combo0)->entry);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      gtk_label_set_mnemonic_widget (GTK_LABEL (label4), propdlg.spin0);
      gtk_label_set_mnemonic_widget (GTK_LABEL (label5), propdlg.spin1);
#if GTK_CHECK_VERSION(2,4,0)
      gtk_label_set_mnemonic_widget (GTK_LABEL (label6), combo);
#else /* not GTK_CHECK_VERSION(2,4,0) */
      gtk_label_set_mnemonic_widget (GTK_LABEL (label6),
                                                    GTK_COMBO (combo)->entry);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      gtk_label_set_mnemonic_widget (GTK_LABEL (label7), spin1);
      gtk_label_set_mnemonic_widget (GTK_LABEL (label8), spin2);

      /* ja:フレームとボックス */
      vbox = gtk_vbox_new (FALSE, SPACING);
      gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), entry0, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), spin0, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label3, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), propdlg.combo0, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label4, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), propdlg.spin0, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label5, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), propdlg.spin1, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label6, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label7, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), spin1, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label8, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), spin2, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      frame0 = gtk_frame_new (_("Video"));
      gtk_container_add (GTK_CONTAINER (frame0), vbox);
      propdlg.video = TRUE;
    }
  else
    {
      frame0 = combo = entry0 = spin0 = spin1 = spin2 = NULL;
      propdlg.video = FALSE;
    }

  if (propcfg->samples_per_sec != 0 && propcfg->channels != 0
                                            && propcfg->bits_per_sample != 0)
    {
      GtkWidget *radio2, *radio3;

      /* ja:コンボボックス */
      glist = NULL;
      for (i = 3; lang[i].text; i++)
        glist = g_list_insert_sorted (glist, (gpointer)_(lang[i].text),
                                                        (GCompareFunc)strcmp);
      for (i = 2; i >= 0; i--)
        glist = g_list_prepend (glist, (gpointer)_(lang[i].text));
#if GTK_CHECK_VERSION(2,4,0)
      propdlg.combo1 = gtk_combo_box_entry_new_text ();
      for (i = g_list_length (glist) - 1; i >= 0; i--)
        gtk_combo_box_prepend_text (GTK_COMBO_BOX (propdlg.combo1),
                                                g_list_nth_data (glist, i));
#else /* not GTK_CHECK_VERSION(2,4,0) */
      propdlg.combo1 = gtk_combo_new ();
      gtk_combo_set_popdown_strings (GTK_COMBO (propdlg.combo1), glist);
      gtk_combo_set_value_in_list (GTK_COMBO (propdlg.combo1), FALSE, FALSE);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      for (i = 0; lang[i].text; i++)
        if (lang[i].id == propcfg->language1)
          {
#if GTK_CHECK_VERSION(2,4,0)
            gtk_entry_set_text
                    (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg.combo1))),
                                                            _(lang[i].text));
#else /* not GTK_CHECK_VERSION(2,4,0) */
            gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (propdlg.combo1)->entry),
                                                            _(lang[i].text));
#endif /* not GTK_CHECK_VERSION(2,4,0) */
            break;
          }
      if (!lang[i].text)
        {
          text = g_strdup_printf ("%X", propcfg->language1);
#if GTK_CHECK_VERSION(2,4,0)
          gtk_entry_set_text
            (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg.combo1))), text);
#else /* not GTK_CHECK_VERSION(2,4,0) */
          gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (propdlg.combo1)->entry),
                                                                        text);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
          g_free (text);
        }
#if GTK_CHECK_VERSION(2,4,0)
      g_signal_connect
                    (G_OBJECT (gtk_bin_get_child (GTK_BIN (propdlg.combo1))),
                        "changed", G_CALLBACK (prop_dialog_changed), &propdlg);
#else /* not GTK_CHECK_VERSION(2,4,0) */
      g_signal_connect (G_OBJECT (GTK_COMBO (propdlg.combo1)->entry),
                        "changed", G_CALLBACK (prop_dialog_changed), &propdlg);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      g_list_free (glist);
      /* ja:エントリー */
      entry1 = gtk_entry_new ();
      if (propcfg->name1)
        gtk_entry_set_text (GTK_ENTRY (entry1), propcfg->name1);
      g_signal_connect (G_OBJECT (entry1), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (entry1), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      /* ja:スピンボタン */
      spin3 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                        (propcfg->priority1, 0, G_MAXSHORT, 1, 5, 0)), 0, 0);
      propdlg.spin2 = gtk_spin_button_new (GTK_ADJUSTMENT (gtk_adjustment_new
                    (propcfg->samples_per_sec, 1, G_MAXINT, 1, 5, 0)), 0, 0);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin3), TRUE);
      gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (propdlg.spin2), TRUE);
      g_signal_connect (G_OBJECT (spin3), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (propdlg.spin2), "activate",
                                G_CALLBACK (prop_dialog_activate), &propdlg);
      g_signal_connect (G_OBJECT (spin3), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      g_signal_connect (G_OBJECT (propdlg.spin2), "changed",
                                G_CALLBACK (prop_dialog_changed), &propdlg);
      /* ja:ラベル */
      label1 = gtk_label_new_with_mnemonic (_("_Name"));
      label2 = gtk_label_new_with_mnemonic (_("_Priority"));
      label3 = gtk_label_new_with_mnemonic (_("_Language"));
      label4 = gtk_label_new_with_mnemonic (_("_Rate"));
      gtk_label_set_mnemonic_widget (GTK_LABEL (label1), entry1);
      gtk_label_set_mnemonic_widget (GTK_LABEL (label2), spin3);
#if GTK_CHECK_VERSION(2,4,0)
      gtk_label_set_mnemonic_widget (GTK_LABEL (label3),
                                gtk_bin_get_child (GTK_BIN (propdlg.combo1)));
#else /* not GTK_CHECK_VERSION(2,4,0) */
      gtk_label_set_mnemonic_widget (GTK_LABEL (label3),
                                            GTK_COMBO (propdlg.combo1)->entry);
#endif /* not GTK_CHECK_VERSION(2,4,0) */
      gtk_label_set_mnemonic_widget (GTK_LABEL (label4), propdlg.spin2);
      /* ja:ラジオボタン */
      radio0 = gtk_radio_button_new_with_mnemonic (NULL, _("_Monophone"));
      radio2 = gtk_radio_button_new_with_mnemonic_from_widget
                                    (GTK_RADIO_BUTTON (radio0), _("_Stereo"));
      radio1 = gtk_radio_button_new_with_mnemonic (NULL, _("_8 bits"));
      radio3 = gtk_radio_button_new_with_mnemonic_from_widget
                                    (GTK_RADIO_BUTTON (radio1), _("_16 bits"));
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                            (propcfg->channels == 1 ? radio0 : radio2), TRUE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                    (propcfg->bits_per_sample == 8 ? radio1 : radio3), TRUE);
      g_signal_connect (G_OBJECT (radio0), "toggled",
                                G_CALLBACK (prop_dialog_toggled), &propdlg);
      g_signal_connect (G_OBJECT (radio1), "toggled",
                                G_CALLBACK (prop_dialog_toggled), &propdlg);
      g_signal_connect (G_OBJECT (radio2), "toggled",
                                G_CALLBACK (prop_dialog_toggled), &propdlg);
      g_signal_connect (G_OBJECT (radio3), "toggled",
                                G_CALLBACK (prop_dialog_toggled), &propdlg);
      /* ja:フレームとボックス */
      vbox = gtk_vbox_new (FALSE, SPACING);
      gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), entry1, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), spin3, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), label3, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), propdlg.combo1, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE,0);
      gtk_box_pack_start (GTK_BOX (hbox), label4, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), propdlg.spin2, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, SPACING);
      gtk_container_set_border_width (GTK_CONTAINER (hbox), SPACING);
      gtk_box_pack_start (GTK_BOX (hbox), radio0, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), radio2, FALSE, FALSE, 0);
      frame1 = gtk_frame_new (_("Channel"));
      gtk_container_add (GTK_CONTAINER (frame1), hbox);
      gtk_box_pack_start (GTK_BOX (vbox), frame1, FALSE, FALSE, 0);

      hbox = gtk_hbox_new (FALSE, SPACING);
      gtk_container_set_border_width (GTK_CONTAINER (hbox), SPACING);
      gtk_box_pack_start (GTK_BOX (hbox), radio1, FALSE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), radio3, FALSE, FALSE, 0);
      frame1 = gtk_frame_new (_("Bit"));
      gtk_container_add (GTK_CONTAINER (frame1), hbox);
      gtk_box_pack_start (GTK_BOX (vbox), frame1, FALSE, FALSE, 0);

      frame1 = gtk_frame_new (_("Audio"));
      gtk_container_add (GTK_CONTAINER (frame1), vbox);
      propdlg.audio = TRUE;
    }
  else
    {
      frame1 = entry1 = radio0 = radio1 = spin3 = NULL;
      propdlg.audio = FALSE;
    }

  /* ja:フレームとボックス */
  vbox = gtk_vbox_new (FALSE, SPACING);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label0, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), propdlg.entry, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  if (frame0 && frame1)
    {
      hbox = gtk_hbox_new (FALSE, SPACING);
      gtk_box_pack_start (GTK_BOX (hbox), frame0, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (hbox), frame1, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
    }
  else
    {
      gtk_box_pack_start (GTK_BOX (vbox),
                                    frame0 ? frame0 : frame1, TRUE, TRUE, 0);
    }

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (propdlg.dialog)->vbox), vbox);

  /* ja:表示 */
  gtk_widget_grab_focus (propdlg.entry);
  gtk_dialog_set_response_sensitive (GTK_DIALOG (propdlg.dialog),
                                                    GTK_RESPONSE_OK, FALSE);
  gtk_widget_show_all (vbox);

  propdlg.ok_sensitive = FALSE;
  propdlg.changed_entry = FALSE;

  result = gtk_dialog_run (GTK_DIALOG (propdlg.dialog)) == GTK_RESPONSE_OK;
  if (result)
    {
      const gchar *text;
      gint i, value;

      if (propdlg.changed_entry)
        {
          gint rate, scale;

          misc_str_to_float (&rate, &scale,
                            gtk_entry_get_text (GTK_ENTRY (propdlg.entry)));
          propcfg->rate = rate;
          propcfg->scale = scale;
        }
      if (propdlg.video)
        {
          /* ja:優先度 */
          propcfg->priority0 = gtk_spin_button_get_value_as_int
                                                    (GTK_SPIN_BUTTON (spin0));
          /* ja:言語 */
          text = gtk_entry_get_text
#if GTK_CHECK_VERSION(2,4,0)
                    (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg.combo0))));
#else /* not GTK_CHECK_VERSION(2,4,0) */
                            (GTK_ENTRY (GTK_COMBO (propdlg.combo0)->entry));
#endif /* not GTK_CHECK_VERSION(2,4,0) */
          for (i = 0; lang[i].text; i++)
            if (g_strcmp (_(lang[i].text), text) == 0)
              {
                propcfg->language0 = lang[i].id;
                break;
              }
          if (!lang[i].text && misc_str_to_val (&value, text, 16, FALSE))
            propcfg->language0 = value;
          /* ja:名前 */
          text = gtk_entry_get_text (GTK_ENTRY (entry0));
          propcfg->name0 = text && text[0] != '\0' ? g_strdup (text) : NULL;
          /* ja:ビデオ */
          propcfg->width = gtk_spin_button_get_value_as_int
                                            (GTK_SPIN_BUTTON (propdlg.spin0));
          propcfg->height = gtk_spin_button_get_value_as_int
                                            (GTK_SPIN_BUTTON (propdlg.spin1));
#if GTK_CHECK_VERSION(2,4,0)
          i = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
          if (i >= 0)
            propcfg->bit_count = bit_count[i].bit;
#else /* not GTK_CHECK_VERSION(2,4,0) */
          text = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry));
          for (i = 0; bit_count[i].text; i++)
            if (g_strcmp (_(bit_count[i].text), text) == 0)
              propcfg->bit_count = bit_count[i].bit;
#endif /* not GTK_CHECK_VERSION(2,4,0) */
          propcfg->x_pixels_per_meter = gtk_spin_button_get_value_as_int
                                                    (GTK_SPIN_BUTTON (spin1));
          propcfg->y_pixels_per_meter = gtk_spin_button_get_value_as_int
                                                    (GTK_SPIN_BUTTON (spin2));
        }
      if (propdlg.audio)
        {
          /* ja:優先度 */
          propcfg->priority1 = gtk_spin_button_get_value_as_int
                                                    (GTK_SPIN_BUTTON (spin3));
          /* ja:言語 */
          text = gtk_entry_get_text
#if GTK_CHECK_VERSION(2,4,0)
                    (GTK_ENTRY (gtk_bin_get_child (GTK_BIN (propdlg.combo1))));
#else /* not GTK_CHECK_VERSION(2,4,0) */
                            (GTK_ENTRY (GTK_COMBO (propdlg.combo1)->entry));
#endif /* not GTK_CHECK_VERSION(2,4,0) */
          for (i = 0; lang[i].text; i++)
            if (g_strcmp (_(lang[i].text), text) == 0)
              {
                propcfg->language1 = lang[i].id;
                break;
              }
          if (!lang[i].text && misc_str_to_val (&value, text, 16, FALSE))
            propcfg->language1 = value;
          /* ja:名前 */
          text = gtk_entry_get_text (GTK_ENTRY (entry1));
          propcfg->name1 = text && text[0] != '\0' ? g_strdup (text) : NULL;
          /* ja:オーディオ */
          propcfg->samples_per_sec = gtk_spin_button_get_value_as_int
                                            (GTK_SPIN_BUTTON (propdlg.spin2));
          propcfg->channels = gtk_toggle_button_get_active
                                        (GTK_TOGGLE_BUTTON (radio0)) ? 1 : 2;
          propcfg->bits_per_sample = gtk_toggle_button_get_active
                                        (GTK_TOGGLE_BUTTON (radio1)) ? 8 : 16;
        }
    }
  gtk_widget_destroy (propdlg.dialog);

  return result;
}
